/*
 * (C) Copyright 2004-2023 Diomidis Spinellis
 *
 * Permission to use, copy, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#include <stdio.h>
#include <stdlib.h>

int
main(int argc, char *argv[])
{
	FILE *f;
	unsigned char b[2] = {0, 0};
	long offset;

	if (argc != 2) {
		fprintf(stderr, "Usage: %s offset\n\n"
			"       FW.EXE File Details                  Offset\n"
			"       -------------------                  ------\n"
			// Joe Hakim
			"1986-12-25                224,128 FW.EXE    0x183bf\n"
			// Alberto Bolado
			"1988-09-12  15:20         252,416 FW.EXE    0x1ce05\n"
			// Diomidis Spinellis 347f:5725
			"1988-09-23  05:53         252,416 FW.EXE    0x1ce05\n"
			// Lothar Kochmann
			"1989-06-30  15:03         232 KB  FW.EXE    0x19A60\n"
			// Dieter Pramberger (Framework III)
			"1990-09-01  22:27         237,696 FW.EXE    0x19A81\n"
			// Gary Gildemeister
			"1990-12-12                237,440 FW.EXE    0x19A53\n"
			// Joan Homar
			"1991-01-31                237,568 FW.EXE    0x19A81\n"
			// Mauro Del Vecchio (Framework IV)
			"1992-08-12  12:00         250,368 FW.EXE    0x18ea1\n"
			// Adri Paardekooper
			"1992-04-09  14:17         237,568 FW.EXE    0x19A66\n"
			// Eduardo Arroyo (Framework IV)
			"1992-10-22                        FW.EXE    0x19A66\n"
			// Milos Kesek
			"1993-02-02  14:22         237,568 FW.EXE    0x19A66\n"
			, argv[0]);
		return 1;
	}
	if ((f = fopen("fw.exe", "r+wb")) == NULL) {
		perror("fw.exe");
		return 1;
	}
	offset = strtol(argv[1], (char **)NULL, 0);
	fprintf(stderr, "The specified offset is 0x%0lx.\n", offset);
	fseek(f, offset, 0);
	fread(b, sizeof(b), 1, f);
	// div bx
	if (b[0] == 0x90 && b[1] == 0x90) {
		fprintf(stderr, "The Framework III/IV executable file fw.exe is already patched\n");
		return 1;
	}
	if (b[0] != 0xf7 || b[1] != 0xf3) {
		fprintf(stderr, "Incompatible Framework III/IV version (bytes=%02x%02x).\n", b[0], b[1]);
		return 1;
	}
	b[0] = b[1] = 0x90;	// nop; nop
	fseek(f, offset, 0);
	fwrite(b, sizeof(b), 1, f);
	fclose(f);
	fprintf(stderr, "The Framework III/IV executable file fw.exe has been patched.\n");
	return 0;
}
